'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _get = require('babel-runtime/helpers/get')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

var WS = require('ws');
var debug = require('debug')('docker-exec-websocket-server:lib:client');
var querystring = require('querystring');
var events = require('events');
var _ = require('lodash');
var assert = require('assert');
var msgcode = require('../lib/messagecodes.js');
var through = require('through');

var DockerExecWebsocketClient = (function (_events$EventEmitter) {
  function DockerExecWebsocketClient(options) {
    _classCallCheck(this, DockerExecWebsocketClient);

    _get(Object.getPrototypeOf(DockerExecWebsocketClient.prototype), 'constructor', this).call(this);
    this.options = _.defaults({}, options, {
      tty: true,
      command: 'sh'
    });
  }

  _inherits(DockerExecWebsocketClient, _events$EventEmitter);

  _createClass(DockerExecWebsocketClient, [{
    key: 'execute',

    /* Makes a client program with unbroken stdin, stdout, stderr streams
     * Is also an EventEmitter with 'exit' event
     *
     * Required options:
     * url parts (hostname port pathname)
     * or url
     * tty: whether or not we expect VT100 style output
     * command: array or string of command to be run in exec
     * not supported yet: ssl cert
     */
    value: function execute() {
      var MAX_OUTSTANDING_BYTES;
      return _regeneratorRuntime.async(function execute$(context$2$0) {
        var _this = this;

        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            this.url = this.options.url + '?' + querystring.stringify({
              tty: this.options.tty ? 'true' : 'false',
              command: this.options.command
            });
            debug(this.url);
            assert(/ws?s:\/\//.test(this.url), 'url required or malformed url input');

            this.socket = new WS(this.url);
            this.socket.binaryType = 'arraybuffer';
            this.socket.onopen = function () {
              debug('socket opened');
              _this.emit('open');
            };

            //set state, state does nothing yet
            this.state = msgcode.pause;
            //stdin stream with pause buffering
            this.stdin = through(function (data) {
              _this.sendMessage(msgcode.stdin, data);
            });

            this.strbuf = through();

            MAX_OUTSTANDING_BYTES = 8 * 1024 * 1024;

            this.outstandingBytes = 0;

            this.strbuf.pause();
            this.strbuf.pipe(through(function (data) {
              _this.outstandingBytes += data.length;
              _this.socket.send(data, { binary: true }, function () {
                _this.outstandingBytes -= data.length;
              });
              if (_this.outstandingBytes > MAX_OUTSTANDING_BYTES) {
                _this.strbuf.pause();
              } else {
                _this.strbuf.resume();
              }
            }));

            this.stdout = through();
            this.stderr = through();

            //Starts out paused so that input isn't sent until server is ready
            this.socket.onmessage = function (messageEvent) {
              _this.messageHandler(messageEvent);
            };
            debug('client executed');

          case 17:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }
  }, {
    key: 'messageHandler',
    value: function messageHandler(messageEvent) {
      var message = new Buffer(new Uint8Array(messageEvent.data));
      debug(message);
      // the first byte is the message code
      switch (message[0]) {
        //pauses the client, causing strbuf to buffer
        case msgcode.pause:
          this.state = msgcode.pause;
          //This stream is created solely for the purposes of pausing, because
          //data will only buffer up in streams using this.queue()
          this.strbuf.pause();
          this.emit('paused');
          break;

        //resumes the client, flushing strbuf
        case msgcode.resume:
          this.state = msgcode.resume;
          this.strbuf.resume();
          this.emit('resumed');
          break;

        case msgcode.stdout:
          this.stdout.write(message.slice(1));
          break;

        case msgcode.stderr:
          this.stderr.write(message.slice(1));
          break;

        //first byte contains exit code
        case msgcode.stopped:
          this.emit('exit', message.readInt8(1));
          this.close();
          break;

        case msgcode.shutdown:
          this.emit('shutdown');
          this.close();
          break;

        case msgcode.error:
          this.emit('error', message.slice(1));
          break;

        default:
          debug('unknown msg code %s', message[0]);
      }
    }
  }, {
    key: 'pause',

    //pauses input coming in from server, useful if you're running out of memory on local (ie probably never)
    value: function pause() {
      this.sendCode(msgcode.pause);
    }
  }, {
    key: 'resume',

    //analogue of pause
    value: function resume() {
      this.sendCode(msgcode.resume);
    }
  }, {
    key: 'sendCode',
    value: function sendCode(code) {
      this.strbuf.write(new Buffer([code]), { binary: true });
    }
  }, {
    key: 'sendMessage',
    value: function sendMessage(code, data) {
      this.strbuf.write(Buffer.concat([new Buffer([code]), new Buffer(data)]), { binary: true });
    }
  }, {
    key: 'close',
    value: function close() {
      var _this2 = this;

      if (!this.strbuf.paused) {
        this.socket.close();
        this.stdin.end();
        this.stdout.end();
        this.stderr.end();
        this.strbuf.end();
      } else {
        this.strbuf.on('drain', function () {
          _this2.socket.close();
          _this2.stdin.end();
          _this2.stdout.end();
          _this2.stderr.end();
          _this2.strbuf.end();
        });
      }
    }
  }]);

  return DockerExecWebsocketClient;
})(events.EventEmitter);

exports['default'] = DockerExecWebsocketClient;
module.exports = exports['default'];